package java.beans.beancontext;

import java.util.HashSet;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Map;
import java.io.ObjectOutputStream;
import java.util.TooManyListenersException;
import java.util.Iterator;
import java.io.Serializable;
import java.util.Locale;
import java.util.ArrayList;
import java.util.HashMap;

public class BeanContextServicesSupport extends BeanContextSupport implements BeanContextServices
{
    private static final long serialVersionUID = -8494482757288719206L;
    protected transient HashMap services;
    protected transient int serializable;
    protected transient BeanContextServicesSupport.BCSSProxyServiceProvider proxy;
    protected transient ArrayList bcsListeners;
    
    public BeanContextServicesSupport(BeanContextServices beanContextServices, Locale locale, boolean b, boolean b2) {
        super(beanContextServices, locale, b, b2);
        this.serializable = 0;
    }
    
    public BeanContextServicesSupport(BeanContextServices beanContextServices, Locale locale, boolean b) {
        this(beanContextServices, locale, b, true);
    }
    
    public BeanContextServicesSupport(BeanContextServices beanContextServices, Locale locale) {
        this(beanContextServices, locale, false, true);
    }
    
    public BeanContextServicesSupport(BeanContextServices beanContextServices) {
        this(beanContextServices, null, false, true);
    }
    
    public BeanContextServicesSupport() {
        this(null, null, false, true);
    }
    
    public void initialize() {
        super.initialize();
        this.services = new HashMap(this.serializable + 1);
        this.bcsListeners = new ArrayList(1);
    }
    
    public BeanContextServices getBeanContextServicesPeer() {
        return (BeanContextServices)this.getBeanContextChildPeer();
    }
    
    @Override
    protected BeanContextSupport.BCSChild createBCSChild(Object o, Object o2) {
        return new BeanContextServicesSupport.BCSSChild(o, o2);
    }
    
    protected BeanContextServicesSupport.BCSSServiceProvider createBCSSServiceProvider(Class clazz, BeanContextServiceProvider beanContextServiceProvider) {
        return new BeanContextServicesSupport.BCSSServiceProvider(clazz, beanContextServiceProvider);
    }
    
    @Override
    public void addBeanContextServicesListener(BeanContextServicesListener beanContextServicesListener) {
        if (beanContextServicesListener == null) {
            throw new NullPointerException("bcsl");
        }
        synchronized (this.bcsListeners) {
            if (this.bcsListeners.contains(beanContextServicesListener)) {
                return;
            }
            this.bcsListeners.add(beanContextServicesListener);
        }
        return;
    }
    
    @Override
    public void removeBeanContextServicesListener(BeanContextServicesListener beanContextServicesListener) {
        if (beanContextServicesListener == null) {
            throw new NullPointerException("bcsl");
        }
        synchronized (this.bcsListeners) {
            if (!this.bcsListeners.contains(beanContextServicesListener)) {
                return;
            }
            this.bcsListeners.remove(beanContextServicesListener);
        }
        return;
    }
    
    @Override
    public boolean addService(Class clazz, BeanContextServiceProvider beanContextServiceProvider) {
        return this.addService(clazz, beanContextServiceProvider, true);
    }
    
    protected boolean addService(Class clazz, BeanContextServiceProvider beanContextServiceProvider, boolean b) {
        if (clazz == null) {
            throw new NullPointerException("serviceClass");
        }
        if (beanContextServiceProvider == null) {
            throw new NullPointerException("bcsp");
        }
        synchronized (BeanContext.globalHierarchyLock) {
            if (this.services.containsKey(clazz)) {
                return false;
            }
            this.services.put(clazz, this.createBCSSServiceProvider(clazz, beanContextServiceProvider));
            if ((beanContextServiceProvider instanceof Serializable)) {
                ++this.serializable;
            }
            if (!b) {
                return true;
            }
            BeanContextServiceAvailableEvent beanContextServiceAvailableEvent = new BeanContextServiceAvailableEvent(this.getBeanContextServicesPeer(), clazz);
            this.fireServiceAdded(beanContextServiceAvailableEvent);
            synchronized (this.children) {
                for (Object next : this.children.keySet()) {
                    if ((next instanceof BeanContextServices)) {
                        ((BeanContextServicesListener)next).serviceAvailable(beanContextServiceAvailableEvent);
                    }
                    else {
                        continue;
                    }
                }
            }
            return true;
        }
    }
    
    @Override
    public void revokeService(Class key, BeanContextServiceProvider obj, boolean b) {
        if (key == null) {
            throw new NullPointerException("serviceClass");
        }
        if (obj == null) {
            throw new NullPointerException("bcsp");
        }
        synchronized (BeanContext.globalHierarchyLock) {
            if (!this.services.containsKey(key)) {
                return;
            }
            if (!((BeanContextServicesSupport.BCSSServiceProvider)this.services.get(key)).getServiceProvider().equals(obj)) {
                throw new IllegalArgumentException("service provider mismatch");
            }
            this.services.remove(key);
            if ((obj instanceof Serializable)) {
                --this.serializable;
            }
            Iterator<Object> bcsChildren = this.bcsChildren();
            while (bcsChildren.hasNext()) {
                ((BeanContextServicesSupport.BCSSChild)bcsChildren.next()).revokeService(key, false, b);
            }
            this.fireServiceRevoked(key, b);
        }
        return;
    }
    
    @Override
    public synchronized boolean hasService(Class key) {
        if (key == null) {
            throw new NullPointerException("serviceClass");
        }
        synchronized (BeanContext.globalHierarchyLock) {
            if (this.services.containsKey(key)) {
                return true;
            }
            BeanContextServices beanContextServices;
            try {
                beanContextServices = (BeanContextServices)this.getBeanContext();
            }
            catch (ClassCastException ex) {
                return false;
            }
            return beanContextServices != null && beanContextServices.hasService(key);
        }
    }
    
    @Override
    public Object getService(BeanContextChild key, Object o, Class key2, Object o2, BeanContextServiceRevokedListener beanContextServiceRevokedListener) throws TooManyListenersException {
        if (key == null) {
            throw new NullPointerException("child");
        }
        if (key2 == null) {
            throw new NullPointerException("serviceClass");
        }
        if (o == null) {
            throw new NullPointerException("requestor");
        }
        if (beanContextServiceRevokedListener == null) {
            throw new NullPointerException("bcsrl");
        }
        BeanContextServices beanContextServicesPeer = this.getBeanContextServicesPeer();
        synchronized (BeanContext.globalHierarchyLock) {
            BeanContextServicesSupport.BCSSChild bcssChild;
            synchronized (this.children) {
                bcssChild = (BeanContextServicesSupport.BCSSChild)this.children.get(key);
            }
            if (bcssChild == null) {
                throw new IllegalArgumentException("not a child of this context");
            }
            BeanContextServicesSupport.BCSSServiceProvider bcssServiceProvider = (BeanContextServicesSupport.BCSSServiceProvider)this.services.get(key2);
            if (bcssServiceProvider != null) {
                BeanContextServiceProvider serviceProvider = bcssServiceProvider.getServiceProvider();
                Object service = serviceProvider.getService(beanContextServicesPeer, o, key2, o2);
                if (service != null) {
                    try {
                        bcssChild.usingService(o, service, key2, serviceProvider, false, beanContextServiceRevokedListener);
                    }
                    catch (TooManyListenersException ex) {
                        serviceProvider.releaseService(beanContextServicesPeer, o, service);
                        throw ex;
                    }
                    catch (UnsupportedOperationException ex2) {
                        serviceProvider.releaseService(beanContextServicesPeer, o, service);
                        throw ex2;
                    }
                    return service;
                }
            }
            if (this.proxy != null) {
                Object service2 = this.proxy.getService(beanContextServicesPeer, o, key2, o2);
                if (service2 != null) {
                    try {
                        bcssChild.usingService(o, service2, key2, this.proxy, true, beanContextServiceRevokedListener);
                    }
                    catch (TooManyListenersException ex3) {
                        this.proxy.releaseService(beanContextServicesPeer, o, service2);
                        throw ex3;
                    }
                    catch (UnsupportedOperationException ex4) {
                        this.proxy.releaseService(beanContextServicesPeer, o, service2);
                        throw ex4;
                    }
                    return service2;
                }
            }
        }
        return null;
    }
    
    @Override
    public void releaseService(BeanContextChild key, Object o, Object o2) {
        if (key == null) {
            throw new NullPointerException("child");
        }
        if (o == null) {
            throw new NullPointerException("requestor");
        }
        if (o2 == null) {
            throw new NullPointerException("service");
        }
        synchronized (BeanContext.globalHierarchyLock) {
            BeanContextServicesSupport.BCSSChild bcssChild;
            synchronized (this.children) {
                bcssChild = (BeanContextServicesSupport.BCSSChild)this.children.get(key);
            }
            if (bcssChild != null) {
                bcssChild.releaseService(o, o2);
            }
            else {
                throw new IllegalArgumentException("child actual is not a child of this BeanContext");
            }
        }
        return;
    }
    
    @Override
    public Iterator getCurrentServiceClasses() {
        return new BeanContextSupport.BCSIterator(this.services.keySet().iterator());
    }
    
    @Override
    public Iterator getCurrentServiceSelectors(Class key) {
        BeanContextServicesSupport.BCSSServiceProvider bcssServiceProvider = (BeanContextServicesSupport.BCSSServiceProvider)this.services.get(key);
        return (bcssServiceProvider != null) ? new BeanContextSupport.BCSIterator(bcssServiceProvider.getServiceProvider().getCurrentServiceSelectors(this.getBeanContextServicesPeer(), key)) : null;
    }
    
    @Override
    public void serviceAvailable(BeanContextServiceAvailableEvent beanContextServiceAvailableEvent) {
        synchronized (BeanContext.globalHierarchyLock) {
            if (this.services.containsKey(beanContextServiceAvailableEvent.getServiceClass())) {
                return;
            }
            this.fireServiceAdded(beanContextServiceAvailableEvent);
            Iterator<Object> iterator;
            synchronized (this.children) {
                iterator = this.children.keySet().iterator();
            }
            while (iterator.hasNext()) {
                Object next = iterator.next();
                if ((next instanceof BeanContextServices)) {
                    ((BeanContextServicesListener)next).serviceAvailable(beanContextServiceAvailableEvent);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    @Override
    public void serviceRevoked(BeanContextServiceRevokedEvent beanContextServiceRevokedEvent) {
        synchronized (BeanContext.globalHierarchyLock) {
            if (this.services.containsKey(beanContextServiceRevokedEvent.getServiceClass())) {
                return;
            }
            this.fireServiceRevoked(beanContextServiceRevokedEvent);
            Iterator<Object> iterator;
            synchronized (this.children) {
                iterator = this.children.keySet().iterator();
            }
            while (iterator.hasNext()) {
                Object next = iterator.next();
                if ((next instanceof BeanContextServices)) {
                    ((BeanContextServicesListener)next).serviceRevoked(beanContextServiceRevokedEvent);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    protected static final BeanContextServicesListener getChildBeanContextServicesListener(Object o) {
        try {
            return (BeanContextServicesListener)o;
        }
        catch (ClassCastException ex) {
            return null;
        }
    }
    
    @Override
    protected void childJustRemovedHook(Object o, BeanContextSupport.BCSChild bcsChild) {
        ((BeanContextServicesSupport.BCSSChild)bcsChild).cleanupReferences();
    }
    
    @Override
    protected synchronized void releaseBeanContextResources() {
        super.releaseBeanContextResources();
        Object[] array;
        synchronized (this.children) {
            if (this.children.isEmpty()) {
                return;
            }
            array = this.children.values().toArray();
        }
        for (int i = 0; i < array.length; ++i) {
            ((BeanContextServicesSupport.BCSSChild)array[i]).revokeAllDelegatedServicesNow();
        }
        this.proxy = null;
    }
    
    @Override
    protected synchronized void initializeBeanContextResources() {
        super.initializeBeanContextResources();
        BeanContext beanContext = this.getBeanContext();
        if (beanContext == null) {
            return;
        }
        try {
            this.proxy = new BeanContextServicesSupport.BCSSProxyServiceProvider((BeanContextServices)beanContext);
        }
        catch (ClassCastException ex) {}
        return;
    }
    
    protected final void fireServiceAdded(Class clazz) {
        this.fireServiceAdded(new BeanContextServiceAvailableEvent(this.getBeanContextServicesPeer(), clazz));
    }
    
    protected final void fireServiceAdded(BeanContextServiceAvailableEvent beanContextServiceAvailableEvent) {
        Object[] array;
        synchronized (this.bcsListeners) {
            array = this.bcsListeners.toArray();
        }
        for (int i = 0; i < array.length; ++i) {
            ((BeanContextServicesListener)array[i]).serviceAvailable(beanContextServiceAvailableEvent);
        }
    }
    
    protected final void fireServiceRevoked(BeanContextServiceRevokedEvent beanContextServiceRevokedEvent) {
        Object[] array;
        synchronized (this.bcsListeners) {
            array = this.bcsListeners.toArray();
        }
        for (int i = 0; i < array.length; ++i) {
            ((BeanContextServiceRevokedListener)array[i]).serviceRevoked(beanContextServiceRevokedEvent);
        }
    }
    
    protected final void fireServiceRevoked(Class clazz, boolean b) {
        BeanContextServiceRevokedEvent beanContextServiceRevokedEvent = new BeanContextServiceRevokedEvent(this.getBeanContextServicesPeer(), clazz, b);
        Object[] array;
        synchronized (this.bcsListeners) {
            array = this.bcsListeners.toArray();
        }
        for (int i = 0; i < array.length; ++i) {
            ((BeanContextServicesListener)array[i]).serviceRevoked(beanContextServiceRevokedEvent);
        }
    }
    
    @Override
    protected synchronized void bcsPreSerializationHook(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.writeInt(this.serializable);
        if (this.serializable <= 0) {
            return;
        }
        int n = 0;
        Iterator<Object> iterator = this.services.entrySet().iterator();
        while (iterator.hasNext() && n < this.serializable) {
            Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
            BeanContextServicesSupport.BCSSServiceProvider obj;
            try {
                obj = (BeanContextServicesSupport.BCSSServiceProvider)entry.getValue();
            }
            catch (ClassCastException ex) {
                continue;
            }
            if ((obj.getServiceProvider() instanceof Serializable)) {
                objectOutputStream.writeObject(entry.getKey());
                objectOutputStream.writeObject(obj);
                ++n;
            }
            else {
                continue;
            }
        }
        if (n != this.serializable) {
            throw new IOException("wrote different number of service providers than expected");
        }
        return;
    }
    
    @Override
    protected synchronized void bcsPreDeserializationHook(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        this.serializable = objectInputStream.readInt();
        for (int i = this.serializable; i > 0; --i) {
            this.services.put(objectInputStream.readObject(), objectInputStream.readObject());
        }
    }
    
    private synchronized void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        this.serialize(objectOutputStream, this.bcsListeners);
    }
    
    private synchronized void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.deserialize(objectInputStream, this.bcsListeners);
    }
    
    protected class BCSSChild extends BeanContextSupport.BCSChild
    {
        private static final long serialVersionUID = -3263851306889194873L;
        private transient HashMap serviceClasses;
        private transient HashMap serviceRequestors;
        
        BCSSChild(Object o, Object o2) {
            super(o, o2);
        }
        
        synchronized void usingService(Object o, Object o2, Class clazz, BeanContextServiceProvider beanContextServiceProvider, boolean b, BeanContextServiceRevokedListener beanContextServiceRevokedListener) throws TooManyListenersException, UnsupportedOperationException {
            Object value = null;
            if (this.serviceClasses == null) {
                this.serviceClasses = new HashMap(1);
            }
            else {
                value = (Object)this.serviceClasses.get(clazz);
            }
            if (value == null) {
                value = new BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef(clazz, beanContextServiceProvider, b);
                this.serviceClasses.put(clazz, value);
            }
            else {
                value.verifyAndMaybeSetProvider(beanContextServiceProvider, b);
                value.verifyRequestor(o, beanContextServiceRevokedListener);
            }
            value.addRequestor(o, beanContextServiceRevokedListener);
            value.addRef(b);
            BeanContextServicesSupport.BCSSChild.BCSSCServiceRef bcsscServiceRef = null;
            Object value2 = null;
            if (this.serviceRequestors == null) {
                this.serviceRequestors = new HashMap(1);
            }
            else {
                value2 = (Object)this.serviceRequestors.get(o);
            }
            if (value2 == null) {
                value2 = new HashMap(1);
                this.serviceRequestors.put(o, value2);
            }
            else {
                bcsscServiceRef = (BeanContextServicesSupport.BCSSChild.BCSSCServiceRef)value2.get(o2);
            }
            if (bcsscServiceRef == null) {
                value2.put(o2, new BeanContextServicesSupport.BCSSChild.BCSSCServiceRef(value, b));
            }
            else {
                bcsscServiceRef.addRef();
            }
        }
        
        synchronized void releaseService(Object o, Object o2) {
            if (this.serviceRequestors == null) {
                return;
            }
            Map<Object, Object> map = (Map)this.serviceRequestors.get(o);
            if (map == null) {
                return;
            }
            BeanContextServicesSupport.BCSSChild.BCSSCServiceRef bcsscServiceRef = (BeanContextServicesSupport.BCSSChild.BCSSCServiceRef)map.get(o2);
            if (bcsscServiceRef == null) {
                return;
            }
            BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef serviceClassRef = bcsscServiceRef.getServiceClassRef();
            int delegated = bcsscServiceRef.isDelegated();
            (delegated ? serviceClassRef.getDelegateProvider() : serviceClassRef.getServiceProvider()).releaseService(BeanContextServicesSupport.this.getBeanContextServicesPeer(), o, o2);
            serviceClassRef.releaseRef((boolean)delegated);
            serviceClassRef.removeRequestor(o);
            if (bcsscServiceRef.release() == 0) {
                map.remove(o2);
                if (map.isEmpty()) {
                    this.serviceRequestors.remove(o);
                    serviceClassRef.removeRequestor(o);
                }
                if (this.serviceRequestors.isEmpty()) {
                    this.serviceRequestors = null;
                }
                if (serviceClassRef.isEmpty()) {
                    this.serviceClasses.remove(serviceClassRef.getServiceClass());
                }
                if (this.serviceClasses.isEmpty()) {
                    this.serviceClasses = null;
                }
            }
            return;
        }
        
        synchronized void revokeService(Class clazz, boolean b, boolean b2) {
            if (this.serviceClasses == null) {
                return;
            }
            BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef obj = (BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef)this.serviceClasses.get(clazz);
            if (obj == null) {
                return;
            }
            Iterator<Object> cloneOfEntries = obj.cloneOfEntries();
            BeanContextServiceRevokedEvent beanContextServiceRevokedEvent = new BeanContextServiceRevokedEvent(BeanContextServicesSupport.this.getBeanContextServicesPeer(), clazz, b2);
            boolean empty = false;
            while (cloneOfEntries.hasNext() && this.serviceRequestors != null) {
                Map.Entry<Object, Object> entry = (Map.Entry)cloneOfEntries.next();
                BeanContextServiceRevokedListener beanContextServiceRevokedListener = (BeanContextServiceRevokedListener)entry.getValue();
                if (b2) {
                    Object key = entry.getKey();
                    Map<Object, Object> map = (Map)this.serviceRequestors.get(key);
                    if (map != null) {
                        Iterator<Object> iterator = map.entrySet().iterator();
                        while (iterator.hasNext()) {
                            BeanContextServicesSupport.BCSSChild.BCSSCServiceRef bcsscServiceRef = (BeanContextServicesSupport.BCSSChild.BCSSCServiceRef)((Map.Entry)iterator.next()).getValue();
                            if (bcsscServiceRef.getServiceClassRef().equals(obj) && b == bcsscServiceRef.isDelegated()) {
                                iterator.remove();
                            }
                            else {
                                continue;
                            }
                        }
                        if (!((empty = map.isEmpty()) == 0)) {
                            this.serviceRequestors.remove(key);
                        }
                    }
                    if (empty != 0) {
                        obj.removeRequestor(key);
                    }
                }
                beanContextServiceRevokedListener.serviceRevoked(beanContextServiceRevokedEvent);
            }
            if (b2 && this.serviceClasses != null) {
                if (obj.isEmpty()) {
                    this.serviceClasses.remove(clazz);
                }
                if (this.serviceClasses.isEmpty()) {
                    this.serviceClasses = null;
                }
            }
            if (this.serviceRequestors != null && this.serviceRequestors.isEmpty()) {
                this.serviceRequestors = null;
            }
            return;
        }
        
        void cleanupReferences() {
            if (this.serviceRequestors == null) {
                return;
            }
            Iterator<Object> iterator = this.serviceRequestors.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
                Object key = entry.getKey();
                Iterator<Object> iterator2 = ((Map)entry.getValue()).entrySet().iterator();
                iterator.remove();
                while (iterator2.hasNext()) {
                    Map.Entry<Object, Object> entry2 = (Map.Entry)iterator2.next();
                    Object key2 = entry2.getKey();
                    BeanContextServicesSupport.BCSSChild.BCSSCServiceRef bcsscServiceRef = (BeanContextServicesSupport.BCSSChild.BCSSCServiceRef)entry2.getValue();
                    BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef serviceClassRef = bcsscServiceRef.getServiceClassRef();
                    BeanContextServiceProvider beanContextServiceProvider = bcsscServiceRef.isDelegated() ? serviceClassRef.getDelegateProvider() : serviceClassRef.getServiceProvider();
                    serviceClassRef.removeRequestor(key);
                    iterator2.remove();
                    while (bcsscServiceRef.release() >= 0) {
                        beanContextServiceProvider.releaseService(BeanContextServicesSupport.this.getBeanContextServicesPeer(), key, key2);
                    }
                }
            }
            this.serviceRequestors = null;
            this.serviceClasses = null;
            return;
        }
        
        void revokeAllDelegatedServicesNow() {
            if (this.serviceClasses == null) {
                return;
            }
            for (BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef obj : new HashSet(this.serviceClasses.values())) {
                if (!obj.isDelegated()) {
                    continue;
                }
                Iterator<Object> cloneOfEntries = obj.cloneOfEntries();
                BeanContextServiceRevokedEvent o = new BeanContextServiceRevokedEvent(BeanContextServicesSupport.this.getBeanContextServicesPeer(), obj.getServiceClass(), true);
                boolean empty = false;
                while (cloneOfEntries.hasNext()) {
                    Map.Entry<Object, Object> entry = (Map.Entry)cloneOfEntries.next();
                    BeanContextServiceRevokedListener beanContextServiceRevokedListener = (BeanContextServiceRevokedListener)entry.getValue();
                    Object key = entry.getKey();
                    Map<Object, Object> map = (Map)this.serviceRequestors.get(key);
                    if (map != null) {
                        Iterator<Object> iterator2 = map.entrySet().iterator();
                        while (iterator2.hasNext()) {
                            BeanContextServicesSupport.BCSSChild.BCSSCServiceRef bcsscServiceRef = (BeanContextServicesSupport.BCSSChild.BCSSCServiceRef)((Map.Entry)iterator2.next()).getValue();
                            if (bcsscServiceRef.getServiceClassRef().equals(obj) && bcsscServiceRef.isDelegated()) {
                                iterator2.remove();
                            }
                            else {
                                continue;
                            }
                        }
                        if (!((empty = map.isEmpty()) == 0)) {
                            this.serviceRequestors.remove(key);
                        }
                    }
                    if (empty != 0) {
                        obj.removeRequestor(key);
                    }
                    beanContextServiceRevokedListener.serviceRevoked(o);
                    if (obj.isEmpty()) {
                        this.serviceClasses.remove(obj.getServiceClass());
                    }
                    else {
                        continue;
                    }
                }
            }
            if (this.serviceClasses.isEmpty()) {
                this.serviceClasses = null;
            }
            if (this.serviceRequestors != null && this.serviceRequestors.isEmpty()) {
                this.serviceRequestors = null;
            }
            return;
        }
        
        class BCSSCServiceClassRef
        {
            Class serviceClass;
            BeanContextServiceProvider serviceProvider;
            int serviceRefs;
            BeanContextServiceProvider delegateProvider;
            int delegateRefs;
            HashMap requestors;
            
            BCSSCServiceClassRef(Class serviceClass, BeanContextServiceProvider beanContextServiceProvider, boolean b) {
                this.requestors = new HashMap(1);
                this.serviceClass = serviceClass;
                if (b) {
                    this.delegateProvider = beanContextServiceProvider;
                }
                else {
                    this.serviceProvider = beanContextServiceProvider;
                }
            }
            
            void addRequestor(Object o, BeanContextServiceRevokedListener beanContextServiceRevokedListener) throws TooManyListenersException {
                BeanContextServiceRevokedListener beanContextServiceRevokedListener2 = (BeanContextServiceRevokedListener)this.requestors.get(o);
                if (beanContextServiceRevokedListener2 != null && !beanContextServiceRevokedListener2.equals(beanContextServiceRevokedListener)) {
                    throw new TooManyListenersException();
                }
                this.requestors.put(o, beanContextServiceRevokedListener);
                return;
            }
            
            void removeRequestor(Object key) {
                this.requestors.remove(key);
            }
            
            void verifyRequestor(Object key, BeanContextServiceRevokedListener obj) throws TooManyListenersException {
                BeanContextServiceRevokedListener beanContextServiceRevokedListener = (BeanContextServiceRevokedListener)this.requestors.get(key);
                if (beanContextServiceRevokedListener != null && !beanContextServiceRevokedListener.equals(obj)) {
                    throw new TooManyListenersException();
                }
                return;
            }
            
            void verifyAndMaybeSetProvider(BeanContextServiceProvider obj, boolean b) {
                BeanContextServiceProvider beanContextServiceProvider;
                if (b) {
                    beanContextServiceProvider = this.delegateProvider;
                    if (beanContextServiceProvider == null || obj == null) {
                        this.delegateProvider = obj;
                        return;
                    }
                }
                else {
                    beanContextServiceProvider = this.serviceProvider;
                    if (beanContextServiceProvider == null || obj == null) {
                        this.serviceProvider = obj;
                        return;
                    }
                }
                if (!beanContextServiceProvider.equals(obj)) {
                    throw new UnsupportedOperationException("existing service reference obtained from different BeanContextServiceProvider not supported");
                }
                return;
            }
            
            Iterator cloneOfEntries() {
                return ((HashMap)this.requestors.clone()).entrySet().iterator();
            }
            
            Iterator entries() {
                return this.requestors.entrySet().iterator();
            }
            
            boolean isEmpty() {
                return this.requestors.isEmpty();
            }
            
            Class getServiceClass() {
                return this.serviceClass;
            }
            
            BeanContextServiceProvider getServiceProvider() {
                return this.serviceProvider;
            }
            
            BeanContextServiceProvider getDelegateProvider() {
                return this.delegateProvider;
            }
            
            boolean isDelegated() {
                return this.delegateProvider != null;
            }
            
            void addRef(boolean b) {
                if (b) {
                    ++this.delegateRefs;
                }
                else {
                    ++this.serviceRefs;
                }
            }
            
            void releaseRef(boolean b) {
                if (b) {
                    if (--this.delegateRefs == 0) {
                        this.delegateProvider = null;
                    }
                }
                else if (--this.serviceRefs <= 0) {
                    this.serviceProvider = null;
                }
            }
            
            int getRefs() {
                return this.serviceRefs + this.delegateRefs;
            }
            
            int getDelegateRefs() {
                return this.delegateRefs;
            }
            
            int getServiceRefs() {
                return this.serviceRefs;
            }
        }
        
        class BCSSCServiceRef
        {
            BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef serviceClassRef;
            int refCnt;
            boolean delegated;
            
            BCSSCServiceRef(BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef serviceClassRef, boolean delegated) {
                this.refCnt = 1;
                this.delegated = false;
                this.serviceClassRef = serviceClassRef;
                this.delegated = delegated;
            }
            
            void addRef() {
                ++this.refCnt;
            }
            
            int release() {
                return --this.refCnt;
            }
            
            BeanContextServicesSupport.BCSSChild.BCSSCServiceClassRef getServiceClassRef() {
                return this.serviceClassRef;
            }
            
            boolean isDelegated() {
                return this.delegated;
            }
        }
    }
    
    protected static class BCSSServiceProvider implements Serializable
    {
        private static final long serialVersionUID = 861278251667444782L;
        protected BeanContextServiceProvider serviceProvider;
        
        BCSSServiceProvider(Class clazz, BeanContextServiceProvider serviceProvider) {
            this.serviceProvider = serviceProvider;
        }
        
        protected BeanContextServiceProvider getServiceProvider() {
            return this.serviceProvider;
        }
    }
    
    protected class BCSSProxyServiceProvider implements BeanContextServiceProvider, BeanContextServiceRevokedListener
    {
        private BeanContextServices nestingCtxt;
        
        BCSSProxyServiceProvider(BeanContextServices nestingCtxt) {
            this.nestingCtxt = nestingCtxt;
        }
        
        @Override
        public Object getService(BeanContextServices beanContextServices, Object o, Class clazz, Object o2) {
            Object service;
            try {
                service = this.nestingCtxt.getService(beanContextServices, o, clazz, o2, this);
            }
            catch (TooManyListenersException ex) {
                return null;
            }
            return service;
        }
        
        @Override
        public void releaseService(BeanContextServices beanContextServices, Object o, Object o2) {
            this.nestingCtxt.releaseService(beanContextServices, o, o2);
        }
        
        @Override
        public Iterator getCurrentServiceSelectors(BeanContextServices beanContextServices, Class clazz) {
            return this.nestingCtxt.getCurrentServiceSelectors(clazz);
        }
        
        @Override
        public void serviceRevoked(BeanContextServiceRevokedEvent beanContextServiceRevokedEvent) {
            Iterator<Object> bcsChildren = BeanContextServicesSupport.this.bcsChildren();
            while (bcsChildren.hasNext()) {
                ((BeanContextServicesSupport.BCSSChild)bcsChildren.next()).revokeService(beanContextServiceRevokedEvent.getServiceClass(), true, beanContextServiceRevokedEvent.isCurrentServiceInvalidNow());
            }
        }
    }
}
